;CHESS sourche code
;by Matrixz (ingegjoestoel@hotmail.com)   
;version 1.0
;compabitable with NESASM 

; Program ordering structure:
;  $8000 : Intro Program
;  $BC00 : Intro Data
;  $C000 : Game Program
;  $E800 : Game Data
;  $EC00 : Commonly Used Subroutines

; RAM ordering structure:
;  $000 : Temporarily RAM
;  $010 : Cursor, position RAM
;  $040 : CHESS board RAM
;  $080 : Chess board Name Table RAM (Used in chess board drawing)

 .inesprg 2  ; Full program
 .ineschr 1  ; One CHR-bank for all the graphics
 .inesmap 16  ; To enable the 4K CHR-ROM page to be used
             ; on both Pattern Tables, we must use a
             ; simple mapper; Bandai 

xA .equ 1
B .equ 2
C .equ 3
D .equ 4
E .equ 5
F .equ 6
G .equ 7
H .equ 8
I .equ 9
J .equ 10
K .equ 11
L .equ 12
M .equ 13
N .equ 14
O .equ 15
P .equ 16
Q .equ 17
R .equ 18
S .equ 19
T .equ 20
U .equ 21
V .equ 22
W .equ 23
xX .equ 24
xY .equ 25
Z .equ 26
sI .equ $30
sG .equ $31
sH .equ $32
sI2 .equ $33
sN .equ $34
n1 .equ $1C
n2 .equ $1D   ; Eneasies text editing

 .bank 0
 .org $8000  ; Set PC to start
program:
 cld
 sei
 ldx #$FF
 txs

 jsr resetmapper
 jsr drawon
 jsr introscreen   ; the intro

resetmapper:
 lda #$01
 sta $8001
 lda #$00       
 sta $8004
 lda #$04
 sta $8005
 rts 
 
introscreen:
 jsr clearnttable
 jsr vv               ; vv=Vait for Vblank. Remember that
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 ldx #$00
intropalget:
 lda intropal,x
 sta $2007
 inx
 cpx #$10
 bne intropalget     ; Set the palette for the intro screen
 
 jsr attableadressing  ; A little routine for adressing the AttribTable
 ldx #$00
introattablechunk:     ; Set the Intro Attribute table up.
 lda introattable,x    ; Each Horizontal line on the screen has
 ldy #$00              ; the same palette data the whole line,
introattablebyte:      ; so we copy 8 bytes.
 sta $2007
 iny
 cpy #$08
 bne introattablebyte 
 inx                   
 cpx #$08              
 bne introattablechunk 
                     
 jsr vv     ; Set up special attribute data
 ldx #$23   ; to the cursor palette.
 ldy #$55
 stx $2006
 lda #$E1
 sta $2006
 sty $2007
 stx $2006
 lda #$E9
 sta $2006
 sty $2007
   
 jsr vv
 lda #$20
 sta $2006
 lda #$78
 sta $2006

 ldy #$00               ; Now, make up the CHESS logo!
intrologoline:         
 lda #$00               ; Notes:
 ldx #$00               ; 16 tiles ($10) is filled blank
intrologoleftfill:      ; after each middle on a line
 sta $2007              ; is drewn. The middle is on 16 tiles
 inx                    ; as well, and is copied directly
 cpx #$10               ; from the data ("chesslogo")
 bne intrologoleftfill
 ldx #$00
intrologochessfill: 
 lda chesslogo,y
 sta $2007
 inx 
 iny
 cpx #$10
 bne intrologochessfill
 cpy #$40
 bne intrologoline

 lda #$21  ; This is a bit more complicated..
 sta $00   ; "word" is a routine that creates
 lda #$48  ; text on the screen using 4 bytes
 sta $01   ; of information that is stored in
 lda #$00  ; RAM from $00-$03 as this:
 sta $02   ;   $00: Upper VRAM memory adress
 lda #$09  ;   $01: Lower VRAM memory adress
 sta $03   ;   $02: The position in text-rom text data lies
 jsr word  ;   $03: How much bytes to read from text-rom
           ; So, $00 and $01 is written to $2006
           ; and $02 is the position in the text data
           ; to start reading from. (Read on for the text format).
           ; The text sat here is actually the "by matrixz" text.

 jsr vv           ; Here we are setting up the bar on the
 lda #$21         ; middle of the screen
 sta $2006
 lda #$A0
 sta $2006
 lda #$77
 ldx #$00
introdrawbar: 
 sta $2007
 inx
 cpx #$20
 bne introdrawbar

 lda #$22    ; Setting up the "White", "Black" and "Texture" text
 sta $00
 lda #$68
 sta $01
 lda #$0A
 sta $02
 lda #$0F
 sta $03
 jsr word

 lda #$22
 sta $00
 lda #$A8
 sta $01
 lda #$0F
 sta $02
 lda #$14
 sta $03
 jsr word

 lda #$22
 sta $00
 lda #$E8
 sta $01
 lda #$14
 sta $02
 lda #$1B
 sta $03
 jsr word

; $00 : Y pixel base
; $01 : X pixel base
; $02 : Current tile
; $03 : Y position counter
; $04 : X position counter
; $08 : Y pixel position
; $09 : X pixel position

 lda #$75   ; This is a code for setting up
 sta $08    ; the "Start" sprite button.
 lda #$40   ; Up there are the RAM used. Examine
 sta $02    ; the code and you will understand it.
 sta $01    ; But i will change this code someday, 
 lda #$00   ; it makes more that 8 sprites on one 
 sta $03    ; scanline.  
 sta $04
 ldx #$00
 lda $01
 sta $09
introspriteline:
introspritetile:
 lda $08
 jsr introspriteput
 lda $02
 jsr introspriteput
 lda #$00
 jsr introspriteput
 lda $09
 jsr introspriteput
 lda $09
 clc
 adc #$08
 sta $09
 inc $02
 inc $04
 lda $04
 cmp #$07
 bne introspritetile
 lda $08
 clc
 adc #$08
 sta $08
 lda $01
 sta $09
 lda $02
 and #%11110000
 clc
 adc #$10
 sta $02
 lda #$00
 sta $04
 inc $03
 lda $03
 cmp #$03
 bne introspriteline
 
 lda #$75  ; Code for the "Load" button
 sta $08
 lda #$7E
 sta $01
 lda #$47
 sta $02
 lda #$00
 sta $03
 sta $04
 ldx #$80
 lda $01
 sta $09
introspriteline2:
introspritetile2:
 lda $08
 jsr introspriteput
 lda $02
 jsr introspriteput
 lda #$01
 jsr introspriteput
 lda $09
 jsr introspriteput
 lda $09
 clc
 adc #$08
 sta $09
 inc $02
 inc $04
 lda $04
 cmp #$07
 bne introspritetile2
 lda $08
 clc
 adc #$08
 sta $08
 lda $01
 sta $09
 lda $02
 and #%11110000
 clc
 adc #$17
 sta $02
 lda #$00
 sta $04
 inc $03
 lda $03
 cmp #$03
 bne introspriteline2

           ;RAM used for menu selection loop:
 lda #$00  ;unhighlighted, human1, woody-texture
 sta $11  ; 'load' button
 sta $12  ; 'white' player mode 
 sta $14  ; texture type   
 sta $15  ; cursor position   
 lda #$01  ;highlighted, human2, stone-texture
 sta $10  ; 'start' button
 lda #$02  ;'computer' player mode
 sta $13  ; 'black' player mode 

 jsr introrefresh  ; Refresh the menu selection items (draw them)
 jsr drawon
 lda #$00
 sta $16

introloop:
 lda #$04
 sta $00
 jsr getbutton
 cmp #$01
 beq ilmodeselect
 lda #$05
 sta $00
 jsr getbutton
 cmp #$01
 beq ilup
 lda #$06
 sta $00
 jsr getbutton
 cmp #$01
 beq ildown
 jmp introloop
ilup:
 ldx $15
 dex
 cpx #$FF
 bne ilOK
 jmp introloop
ildown:
 ldx $15
 inx
 cpx #$05
 bne ilOK
 jmp introloop

ilOK:
 stx $15
 jsr introrefresh
 lda #$70
 sta $00
 jsr pause
 jmp introloop

ilmodeselect:
 lda $15
 cmp #$00
 bne ilmodenot0
 lda $14
 cmp #$00
 bne ilstartasn0
 lda #$AA         
 sta $00
 jmp ilstartasf
ilstartasn0:
 cmp #$01
 bne ilstartasn1
 lda #$00          
 sta $00
 jmp ilstartasf
ilstartasn1:
 cmp #$02
 bne ilstartasn2
 lda #$FF       
 sta $00
 jmp ilstartasf
ilstartasn2:
 lda #$55         
 sta $00
ilstartasf:
 ldx #$00
chessramreset:
 lda boardsetup,x
 sta $40,x
 inx
 cpx #$40
 bne chessramreset
 jmp game
ilmodenot0:
 cmp #$01
 bne ilmodenot1
 jmp loadgame
ilmodenot1:
 cmp #$02
 bne ilmodenot2
 inc $12
 lda $12
 cmp #$03
 bne ilwhiteOK
 lda #$00
 sta $12
ilwhiteOK:
 jsr introrefresh
 lda #$70
 sta $00
 jsr pause
 jmp introloop
ilmodenot2:
 cmp #$03
 bne ilmodenot3
 inc $13
 lda $13
 cmp #$03
 bne ilblackOK
 lda #$00
 sta $13
ilblackOK:
 jsr introrefresh
 lda #$70
 sta $00
 jsr pause
 jmp introloop
ilmodenot3:
 inc $14
 lda $14
 cmp #$04
 bne iltexOK
 lda #$00
 sta $14
iltexOK:
 jsr introrefresh
 lda #$70
 sta $00
 jsr pause
 jmp introloop

loadgame:
 lda #$00
 sta $19

 lda #$05
 sta $8001
 lda #$04
 sta $8002

 jsr clearnttable
 jsr clearsprite
 
 lda #$20
 sta $00
 lda #$41
 sta $01
 lda #$1B
 sta $02
 lda #$24
 sta $03
 jsr word

 jsr vv         
 lda #$20   
 sta $2006
 lda #$60
 sta $2006
 lda #$7B
 ldx #$00
lgdrawbar: 
 sta $2007
 inx
 cpx #$20
 bne lgdrawbar

 jsr vv
 lda #$20
 sta $2006
 lda #$80 
 sta $2006
 jsr drawbox

 lda #$21
 sta $00
 lda #$01
 sta $01
 lda #$0A
 sta $02
 lda #$0F
 sta $03
 jsr word

 lda #$21
 sta $00
 lda #$41
 sta $01
 lda #$0F
 sta $02
 lda #$14
 sta $03
 jsr word

 lda #$21
 sta $00
 lda #$81
 sta $01
 lda #$14
 sta $02
 lda #$1B
 sta $03
 jsr word

 lda #$21
 sta $00
 lda #$C1
 sta $01
 lda #$24
 sta $02
 lda #$28
 sta $03
 jsr word

 ldx #$00
loadtransprgset: 
 lda loadtransprg,x
 sta $0200,x
 inx
 cpx #$09
 bne loadtransprgset

 lda $19
 clc
 adc #$60
 sta $0202
 
 ldx #$00
loadtransfer:
 jmp $0200
loadtransfer2:
 inx
 cpx #$00
 bne loadtransfer

 jmp loop

loadtransprg:
 lda $6000,x
 sta $0100,x
 jmp loadtransfer2

introspriteput:   ; A much-used routine in some of
 stx $2003        ; the code above
 inx
 sta $2004
 rts

introrefresh:
 lda $15
 cmp #$00
 bne introcurnot0
 lda #$01 
 sta $10
 lda #$00
 sta $11
 jmp introrefnext0
introcurnot0:
 lda $15
 cmp #$01
 bne introcurnot1
 lda #$00
 sta $10
 lda #$01
 sta $11
 jmp introrefnext0
introcurnot1:
 lda #$00
 sta $10
 sta $11

introrefnext0:
 jsr vv
 lda #$3F
 sta $2006
 lda #$10
 sta $2006

 lda $10              ; Set up the sprite palette
 cmp #$00             ; for the "Start" button.
 bne introhighlight1  ; If it is set, it's
 ldx #$00             ; highlighted
introrefpalget11: 
 lda introsprpal,x
 sta $2007
 inx
 cpx #$04
 bne introrefpalget11
 jmp introrefnext1
introhighlight1:
 ldx #$04
introrefpalget12: 
 lda introsprpal,x
 sta $2007
 inx
 cpx #$08
 bne introrefpalget12

introrefnext1:
 lda $11              ; Set up sprite palette
 cmp #$00             ; for "Load" button
 bne introhighlight2  
 ldx #$00
introrefpalget21: 
 lda introsprpal,x
 sta $2007
 inx
 cpx #$04
 bne introrefpalget21
 jmp introrefnext1B
introhighlight2:
 ldx #$04
introrefpalget22: 
 lda introsprpal,x
 sta $2007
 inx
 cpx #$08
 bne introrefpalget22

introrefnext1B:    ; Code for drawing position of cursor      
 ldx #$66               ; LOW VRAM adress 
 ldy #$00               ; Compare value
introcurset:
 jsr vv
 lda #$22               ; HIGH VRAM adress
 sta $2006              
 stx $2006
 lda $15                ; Cursor position
 clc
 sbc #$01               ; Subtract cursor cases 0 and 1 ("Start
 sty $00                ; and "Load") buttons
 cmp $00                ; Compare cursor position -2 with counter
 beq introcurhere       ; BEQ if they are equal.
 lda #$00               ; If not, draw it blank
 sta $2007
 jmp introcurfinishdraw
introcurhere:
 lda #$62               ; $62 = Cursor tile
 sta $2007
introcurfinishdraw:
 txa                    ; Make operation possible
 clc
 adc #$40               ; Add $40 to LOW VRAM adress
 tax                    ; (+2 lines)
 iny
 cpy #$03               ; Do on three lines
 bne introcurset
 
 jsr vv
 lda #$23
 sta $2006
 lda #$E4
 sta $2006
 lda $12
 sta $00
 cmp #$00
 bne introptypeWcolNOT0
 lda #$F0
 jmp introptypeWcoldraw
introptypeWcolNOT0:
 lda $00
 cmp #$01
 bne introptypeWcolNOT1
 lda #$50
 jmp introptypeWcoldraw
introptypeWcolNOT1:
 lda $00
 cmp #$02
 bne introptypeWcolNOT2
 lda #$A0
 jmp introptypeWcoldraw
introptypeWcolNOT2:
 jmp introrefnext2
introptypeWcoldraw: 
 ldx #$00
introptypeWcolloop:
 sta $2007
 inx
 cpx #$03
 bne introptypeWcolloop

introrefnext2:
 lda $13
 sta $00
 cmp #$00
 bne introptypeBcolNOT0
 lda #$0F
 jmp introptypeBcolset
introptypeBcolNOT0:
 lda $00
 cmp #$01
 bne introptypeBcolNOT1
 lda #$05
 jmp introptypeBcolset
introptypeBcolNOT1:
 lda $00
 cmp #$02
 bne introptypeBcolNOT2
 lda #$0A
 jmp introptypeBcolset
introptypeBcolNOT2:
 jmp introrefnext3
introptypeBcolset:
 sta $01

introrefnext3:
 jsr vv
 lda #$23
 sta $2006
 lda #$EC
 sta $2006
 lda $14
 sta $00
 cmp #$00
 bne introtexcolNOT0
 lda #$A0
 jmp introtexcolset
introtexcolNOT0:
 lda $00
 cmp #$01
 bne introtexcolNOT1
 lda #$00
 jmp introtexcolset
introtexcolNOT1:
 lda $00
 cmp #$02
 bne introtexcolNOT2
 lda #$F0
 jmp introtexcolset
introtexcolNOT2:       
 lda $00
 cmp #$03
 bne introtexcolNOT3   
 lda #$50
 jmp introtexcolset
introtexcolNOT3:
 jmp introrefnext4
introtexcolset: 
 clc
 adc $01
 ldx #$00
introcolloop:
 sta $2007
 inx
 cpx #$03
 bne introcolloop

introrefnext4: 
 jsr vv             ; This sets up text for the "White" and
 lda #$22           ; "Black" player modes using a subroutine
 sta $2006          ; that sets it to "Player 1", "Player 2",
 lda #$71           ; or "Computer". Palette setting code
 sta $2006          ; not included.
 lda $12
 jsr introptypeset

 jsr vv
 lda #$22
 sta $2006
 lda #$B1
 sta $2006
 lda $13
 jsr introptypeset

 jsr vv
 lda #$22
 sta $2006
 lda #$F1
 sta $2006
 lda $14
 sta $00
 cmp #$00
 bne introtextypNOT0
 ldx #$18
 lda #$1D
 sta $02
 jmp introtextypset
introtextypNOT0:
 lda $00
 cmp #$01
 bne introtextypNOT1
 ldx #$1D
 lda #$22
 sta $02
 jmp introtextypset
introtextypNOT1:
 lda $00
 cmp #$02
 bne introtextypNOT2
 ldx #$22
 lda #$27
 sta $02
 jmp introtextypset
introtextypNOT2:
 lda $00
 cmp #$03
 bne introtextypNOT3
 ldx #$27
 lda #$2B
 sta $02
 jmp introtextypset
introtextypNOT3:
 jmp introrefnext5
introtextypset:
 jsr introptypewrite

introrefnext5:
 rts

introptypeset:
 sta $01
 cmp #$00
 bne introptypeNOT0
 ldx #$00
 lda #$08
 sta $02
 jmp introptypewrite
introptypeNOT0:
 lda $01
 cmp #$01
 bne introptypeNOT1
 ldx #$08
 lda #$10
 sta $02
 jmp introptypewrite
introptypeNOT1:
 lda $01
 cmp #$02
 bne introptypeNOT2
 ldx #$10
 lda #$18
 sta $02
 jmp introptypewrite
introptypeNOT2:
 rts

introptypewrite: 
 lda playertypetext,x
 sta $2007
 inx
 cpx $02
 bne introptypewrite
 rts

 .bank 1
 .org $BC00
intropal: .db $0F,$27,$18,$08,$0F,$26,$16,$06,$0F,$30,$10,$00,$0F,$21,$11,$01
introsprpal: .db $0F,$0F,$11,$01,$0F,$38,$21,$11
introattable: .db $00,$00,$A5,$FF,$AA,$FA,$00,$00
chesslogo: 
  .db $40,$41,$42,$43,$00,$45,$46,$47,$48,$46,$47,$48,$46,$47,$48,$00
  .db $50,$00,$52,$53,$54,$55,$56,$57,$58,$56,$57,$58,$56,$57,$58,$00
  .db $50,$00,$52,$63,$64,$65,$56,$67,$64,$60,$64,$65,$60,$64,$65,$00
  .db $70,$71,$72,$73,$00,$75,$76,$66,$74,$51,$61,$44,$51,$61,$44,$00
playertypetext: .db H,U,M,xA,N,0,n1,0,H,U,M,xA,N,0,n2,0,C,O,M,P,U,T,E,R,S,T,O,N,E,W,O,O,D,xY,B,L,U,E,0,R,E,D,0,0       
boardsetup: 
  .db $0C,$0A,$0B,$0D,$0E,$0B,$0A,$0C 
  .db $09,$09,$09,$09,$09,$09,$09,$09
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $01,$01,$01,$01,$01,$01,$01,$01
  .db $04,$02,$03,$05,$06,$03,$02,$04




 .bank 2
 .org $C000
game:
 jsr chessboardreset
 jsr chessboardset
 jsr chessboarddraw

Wgameturn:
 ldy #$00
 sty $18

 lda $20
 sta $26
 lda $21
 sta $27
 lda $24
 sta $20
 lda $25
 sta $21
 jmp gamelpp
Bgameturn: 
 ldy #$01
 sty $18

 lda $20
 sta $24
 lda $21
 sta $25
 lda $26
 sta $20
 lda $27
 sta $21
 jmp gamelpp

gamelpp:
 lda $12,y
 cmp #$00
 bne ptnot0
 jmp P1
ptnot0:
 cmp #$01
 bne ptnot1
 jmp P2
ptnot1:
 jmp CPU

P1:
 lda #$00
 sta $16
 lda #$01
 sta $0A
 lda #$21
 sta $0B
 jmp gamelp

P2:
 lda #$01
 sta $16
 lda #$0F
 sta $0A
 lda #$16
 sta $0B
 jmp gamelp

gamelp:
 jsr chessboardset
 jsr chessboarddraw

 jsr vv
 lda #$3F
 sta $2006
 lda #$11
 sta $2006

 lda #$00
 sta $0C

 ldx #$00
pcursprpalw:
 lda $0A,x
 sta $2007
 inx
 cpx #$03
 bne pcursprpalw

 jsr cursprallset
 jmp gameloopJ

gameloop:
 lda #$01
 sta $00
 jsr getbutton
 cmp #$01
 beq charpickJ
 lda #$03
 sta $00
 jsr getbutton
 cmp #$01
 beq gamemenuJ1
 lda #$05
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilup
 lda #$06
 sta $00
 jsr getbutton
 cmp #$01
 beq gameildown
 lda #$07
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilleft
 lda #$08
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilright
 jmp gameloop
gamemenuJ1:
 jmp gamemenu
charpickJ:
 jmp charpick
gameilup:
 lda #$00
 sta $02
 lda $20
 tax
 dex
 txa
 cmp #$FF
 bne gameilYJ
 jmp gameloop
gameildown:
 lda #$01
 sta $02
 lda $20        
 tax
 inx
 txa
 cmp #$08
 bne gameilYJ
 jmp gameloop
gameilleft:
 lda #$00
 sta $02
 lda $21
 tax
 dex
 txa
 cmp #$FF
 bne gameilXJ
 jmp gameloop
gameilright:
 lda #$01
 sta $02                                
 lda $21
 tax
 inx
 txa
 cmp #$08
 bne gameilXJ
 jmp gameloop

gameilXJ:
 jsr gameilX
 jmp gameloop
gameilYJ:
 jsr gameilY
 jmp gameloop

charpick:
 lda #$00
 ldx #$00
charpickYcheck:
 clc
 adc #$08
 inx
 cpx $20
 bne charpickYcheck
 ldx #$00
charpickXcheck:
 clc
 adc #$01
 inx
 cpx $21
 bne charpickXcheck
 tax
 lda $40,x
 and #%00001000
 lsr A
 lsr A
 lsr A
 cmp $18
 bne gameloopJ
charpiccheck:
 lda $40,x
 cmp #$00
 beq gameloopJ

game2:
 lda $20
 sta $22
 lda $21
 sta $23
 
gameloop2:
 jsr mcursprallset
 lda #$01
 sta $00
 jsr getbutton
 cmp #$01
 beq cmcontrolJ
 lda #$02
 sta $00
 jsr getbutton
 cmp #$01
 beq gameloopJ
 lda #$03
 sta $00
 jsr getbutton
 cmp #$01
 beq gamemenuJ2
 lda #$05
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilup2
 lda #$06
 sta $00
 jsr getbutton
 cmp #$01
 beq gameildown2
 lda #$07
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilleft2
 lda #$08
 sta $00
 jsr getbutton
 cmp #$01
 beq gameilright2
 jmp gameloop2
gamemenuJ2:
 jmp gameloop
cmcontrolJ:
 jmp cmcontrol
gameloopJ:
 lda #$00
 ldx #$00 
mcsprclear: 
 stx $2003
 sta $2004
 inx
 cpx #$10
 bne mcsprclear
 jmp gameloop
gameilup2:
 lda #$00
 sta $02
 lda $22
 tax
 dex
 txa
 cmp #$FF
 bne gameilY2J
 jmp gameloop2
gameildown2:
 lda #$01
 sta $02
 lda $22        
 tax
 inx
 txa
 cmp #$08
 bne gameilY2J
 jmp gameloop2
gameilleft2:
 lda #$00
 sta $02
 lda $23
 tax
 dex
 txa
 cmp #$FF
 bne gameilX2J
 jmp gameloop2
gameilright2:
 lda #$01
 sta $02                                
 lda $23
 tax
 inx
 txa
 cmp #$08
 bne gameilX2J
 jmp gameloop2
gameilX2J:
 jsr gameilX2
 jmp gameloop2
gameilY2J:
 jsr gameilY2
 jmp gameloop2

cmcontrol:
 lda $18
 cmp #$00
 beq whitecmcontrol
 cmp #$01
 beq blackcmcontrolJ
blackcmcontrolJ:
 jmp blackcmcontrol

whitecmcontrol:
 lda #$00
 ldx #$00
wcpcYcheck:
 clc
 adc #$08
 inx
 cpx $20
 bne wcpcYcheck
 ldx #$00
wcpcXcheck:
 clc
 adc #$01
 inx
 cpx $21
 bne wcpcXcheck
 sta $00
 tax
 lda $40,x
 sta $01
 lda #$00
 ldx #$00
wcmcYcheck:
 clc
 adc #$08
 inx
 cpx $22
 bne wcmcYcheck
 ldx #$00
wcmcXcheck:
 clc
 adc #$01
 inx
 cpx $23
 bne wcmcXcheck
 sta $02
 tax
 lda $40,x
 sta $03
 jmp whitecmc2

; RAM Table:
; $00: Selected Character Position
; $01: Selected Character Data
; $02: Destination Position
; $03: Destination Data
; $20: Selected Character Y Position
; $21: Selected Character X Position
; $22: Destination Y Point
; $23: Destination X Point

whitegameloop2J1:
 jmp gameloop2
whitecNOT1J:
 jmp whitecNOT1 

whitecmc2:
 lda $20
 cmp $22
 bne whitecmc3
   lda $21
   cmp $23
   bne whitecmc3
     jmp gameloop2
  
whitecmc3:
 lda $03
 and #%0000111
 cmp #$00
 beq whitecmc4
 
 lda $03
 and #%00001000
 lsr A
 lsr A
 lsr A
 cmp $18
 beq whitegameloop2J1
 
whitecmc4:
 lda $03
 cmp #$0E
 beq whitegameloop2J1

 lda $01 
 cmp #$01
 bne whitecNOT1J

  lda $20
  clc
  sbc $22
  cmp #$01
  beq WC1C1A
    lda $20
    clc
    sbc $22
    cmp #$00
    beq WC1C1B
      jmp gameloop2
  WC1C1A:
   lda $20
   cmp #$06
   bne whitegameloop2J1
     lda $21
     clc
     sbc $23
     cmp #$FF
     bne whitegameloop2J1
       lda $02
       clc
       adc #$08
       tax
       lda $40,x
       and #%00000111
       cmp #$00
       bne whitegameloop2J1
         ldx $02
         lda $40,x
         and #%00000111
         cmp #$00
         bne whitegameloop2J2
           jmp charmove
  WC1C1B:
   lda $21
   clc
   sbc $23
   cmp #$FE
   beq WC1C1AA
     cmp #$FF 
     beq WC1C1AB
       cmp #$00
       beq WC1C1AA
         jmp gameloop2
     WC1C1AA:
      lda $03
      and #%00000111
      cmp #$00
      beq whitegameloop2J2
        lda $03
        and #%00001000
        cmp #00
        beq whitegameloop2J2
          lda $22
          cmp #$00
          bne whitecharmoveJ1
            lda #$05
            sta $01
            jmp charmove
      WC1C1AB:
       lda $03
       and #%00000111
       cmp #$00
       bne whitegameloop2J2
         jmp charmove

whitecharmoveJ1:
 jmp charmove
whitegameloop2J2:
 jmp gameloop2

whitecNOT1:
 cmp #$02
 bne whitecNOT2

  ldx #$00
 WC2C1L:
  lda WC2Yposdat,x
  sta $08
  lda WC2Xposdat,x
  sta $09

  lda $20
  clc
  adc #$03
  clc
  sbc $08
  cmp $22
  bne WC2NA
    lda $21
    clc
    adc #$07
    clc
    sbc $09
    cmp $23
    bne WC2NA
      jmp charmove
WC2NA:
  inx
  cpx #$08
  bne WC2C1L
  jmp gameloop2

WC2Yposdat:
 .db $04
 .db $04
 .db $03
 .db $03
 .db $01
 .db $01
 .db $00
 .db $00

WC2Xposdat: 
 .db $07
 .db $05
 .db $08 
 .db $04
 .db $08
 .db $04
 .db $05
 .db $07

whitegameloop2J3:
 jmp gameloop2
whitecNOT3J:
 jmp whitecNOT3
whitecNOT2:
 cmp #$03
 bne whitecNOT3J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  bpl WC3U
    bmi WC3D
      jmp gameloop2
  WC3U:
   lda $21 
   cmp $23
   bpl WC3UL
     bmi WC3UR
       jmp gameloop2
     WC3UL:
      dec $08
      dec $09
      lda $08
      cmp #$FF
      beq whitegameloop2J3
      lda $09
      cmp #$FF
      beq whitegameloop2J3
      jsr WC3C
      jmp WC3UL
     WC3UR:
      dec $08
      inc $09
      lda $08
      cmp #$FF
      beq whitegameloop2J3
      lda $09
      cmp #$08
      beq whitegameloop2J3
      jsr WC3C
      jmp WC3UR
  WC3D:
   lda $21 
   cmp $23
   bpl WC3DL
     bmi WC3DR
       jmp gameloop2
     WC3DL:
      inc $08
      dec $09
      lda $08
      cmp #$08
      beq whitegameloop2J3
      lda $09
      cmp #$FF
      beq whitegameloop2J4
      jsr WC3C
      jmp WC3DL
     WC3DR:
      inc $08
      inc $09
      lda $08
      cmp #$08
      beq whitegameloop2J4
      lda $09
      cmp #$08
      beq whitegameloop2J4
      jsr WC3C
      jmp WC3DR
       
  WC3C:
   lda $08
   cmp $22
   bne WC3CN
     lda $09
     cmp $23
     bne WC3CN
       jmp charmove
     WC3CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne whitegameloop2J4
      rts

whitegameloop2J4:
 jmp gameloop2
whitecNOT4J:
 jmp whitecNOT4
whitecNOT3:
 cmp #$04
 bne whitecNOT4J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  beq WC4YEQ
  lda $21
  cmp $23
  beq WC4XEQ
    jmp gameloop2
 WC4YEQ:
  lda $21
  cmp $23
  bpl WC4YL
    bmi WC4YR
      jmp gameloop2
  WC4YL:
   dec $09
   lda $09
   cmp #$FF
   beq whitegameloop2J4
   jsr WC4C
   jmp WC4YL
  WC4YR:
   inc $09
   lda $09
   cmp #$08
   beq whitegameloop2J4
   jsr WC4C
   jmp WC4YR
 WC4XEQ:
  lda $20
  cmp $22
  bpl WC4XU
    bmi WC4XD
      jmp gameloop2
  WC4XU:
   dec $08
   lda $08
   cmp #$FF
   beq whitegameloop2J4
   jsr WC4C
   jmp WC4XU
  WC4XD:
   inc $08
   lda $08
   cmp #$08
   beq whitegameloop2J4
   jsr WC4C
   jmp WC4XD
  
  WC4C:
   lda $08
   cmp $22
   bne WC4CN
     lda $09
     cmp $23
     bne WC4CN
       jmp charmove
     WC4CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne whitegameloop2J5
      rts

whitegameloop2J5:
 jmp gameloop2
whitecNOT5J:
 jmp whitecNOT5

whitecNOT4:
 cmp #$05
 bne whitecNOT5J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  beq WC5YEQ
  lda $21
  cmp $23
  beq WC5XEQ
  lda $20
  cmp $22
  bpl WC5U
    bmi WC5DJ
      jmp gameloop2
WC5DJ:
 jmp WC5D

 WC5YEQ:
  lda $21
  cmp $23
  bpl WC5YL
    bmi WC5YR
      jmp gameloop2
  WC5YL:
   dec $09
   lda $09
   cmp #$FF
   beq whitegameloop2J5
   jsr WC5C
   jmp WC5YL
  WC5YR:
   inc $09
   lda $09
   cmp #$08
   beq whitegameloop2J5
   jsr WC5C
   jmp WC5YR
 WC5XEQ:
  lda $20
  cmp $22
  bpl WC5XU
    bmi WC5XD
      jmp gameloop2
  WC5XU:
   dec $08
   lda $08
   cmp #$FF
   beq whitegameloop2J5
   jsr WC5C
   jmp WC5XU
  WC5XD:
   inc $08
   lda $08
   cmp #$08
   beq whitegameloop2J6
   jsr WC5C
   jmp WC5XD
  
whitegameloop2J6:
  jmp gameloop2

 WC5U:
  lda $21 
  cmp $23
  bpl WC5UL
    bmi WC5UR
      jmp gameloop2
    WC5UL:
      dec $08
      dec $09
      lda $08
      cmp #$FF
      beq whitegameloop2J6
      lda $09
      cmp #$FF
      beq whitegameloop2J6
      jsr WC5C
      jmp WC5UL
    WC5UR:
      dec $08
      inc $09
      lda $08
      cmp #$FF
      beq whitegameloop2J6
      lda $09
      cmp #$08
      beq whitegameloop2J6
      jsr WC5C
      jmp WC5UR
 WC5D:
  lda $21 
  cmp $23
  bpl WC5DL
    bmi WC5DR
      jmp gameloop2
    WC5DL:
     inc $08
     dec $09
     lda $08
     cmp #$08
     beq whitegameloop2J6
     lda $09
     cmp #$FF
     beq whitegameloop2J6
     jsr WC5C
     jmp WC5DL
    WC5DR:
     inc $08
     inc $09
     lda $08
     cmp #$08
     beq whitegameloop2J6
     lda $09
     cmp #$08
     beq whitegameloop2J6
     jsr WC5C
     jmp WC5DR
  
  WC5C:
   lda $08
   cmp $22
   bne WC5CN
     lda $09
     cmp $23
     bne WC5CN
       jmp charmove
     WC5CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne whitegameloop2J7
      rts
 
whitegameloop2J7:
  jmp gameloop2
whitecNOT5:  
 cmp #$06
 bne whitecharmoveJ2

   lda $20
   clc
   sbc $22
   cmp #$FE
   beq WC7YA
   cmp #$FF
   beq WC7YA
   cmp #$00
   beq WC7YA
     jmp gameloop2
     
   WC7YA:
     lda $21
     clc
     sbc $23
     cmp #$FE
     beq whitecharmoveJ2
     cmp #$FF 
     beq whitecharmoveJ2
     cmp #$00
     beq whitecharmoveJ2
       jmp gameloop2

whitecharmoveJ2:
 jmp charmove

blackcmcontrol:
 lda #$00
 ldx #$00
bcpcYcheck:
 clc
 adc #$08
 inx
 cpx $20
 bne bcpcYcheck
 ldx #$00
bcpcXcheck:
 clc
 adc #$01
 inx
 cpx $21
 bne bcpcXcheck
 sta $00
 tax
 lda $40,x
 sta $01
 lda #$00
 ldx #$00
bcmcYcheck:
 clc
 adc #$08
 inx
 cpx $22
 bne bcmcYcheck
 ldx #$00
bcmcXcheck:
 clc
 adc #$01
 inx
 cpx $23
 bne bcmcXcheck
 sta $02
 tax
 lda $40,x
 sta $03
 jmp blackcmc2

; RAM Table:
; $00: Selected Character Position
; $01: Selected Character Data
; $02: Destination Position
; $03: Destination Data
; $20: Selected Character Y Position
; $21: Selected Character X Position
; $22: Destination Y Point
; $23: Destination X Point

blackgameloop2J1:
 jmp gameloop2
blackcNOT1J:
 jmp blackcNOT1 

blackcmc2:
 lda $20
 cmp $22
 bne blackcmc3
   lda $21
   cmp $23
   bne blackcmc3
     jmp gameloop2
  
blackcmc3:
 lda $03
 and #%0000111
 cmp #$00
 beq blackcmc4
 
 lda $03
 and #%00001000
 lsr A
 lsr A
 lsr A
 cmp $18
 beq blackgameloop2J1
 
blackcmc4:
 lda $03
 cmp #$07
 beq blackgameloop2J1

 lda $01 
 cmp #$09
 bne blackcNOT1J

  lda $20
  clc
  sbc $22
  cmp #$FE
  beq BC1C1A
    lda $20
    clc
    sbc $22
    cmp #$FD
    beq BC1C1B
      jmp gameloop2
  BC1C1A:
   lda $20
   cmp #$01
   bne blackgameloop2J1
     lda $21
     clc
     sbc $23
     cmp #$FF
     bne blackgameloop2J1
       lda $02
       clc
       adc #$08
       tax
       lda $40,x
       and #%00000111
       cmp #$00
       bne blackgameloop2J1
         ldx $02
         lda $40,x
         and #%00000111
         cmp #$00
         bne blackgameloop2J2
           jmp charmove
  BC1C1B:
   lda $21
   clc
   sbc $23
   cmp #$FE
   beq BC1C1AA
     cmp #$FF 
     beq BC1C1AB
       cmp #$00
       beq BC1C1AA
         jmp gameloop2
     BC1C1AA:
      lda $03
      and #%00000111
      cmp #$00
      beq blackgameloop2J2
        lda $03
        and #%00001000
        cmp #$01
        beq blackgameloop2J2
          lda $22
          cmp #$00
          bne blackcharmoveJ1
            lda #$0D
            sta $01
            jmp charmove
      BC1C1AB:
       lda $03
       and #%00000111
       cmp #$00
       bne blackgameloop2J2
         jmp charmove

blackcharmoveJ1:
 jmp charmove
blackgameloop2J2:
 jmp gameloop2

blackcNOT1:
 cmp #$0A
 bne blackcNOT2

  ldx #$00
 BC2C1L:
  lda BC2Yposdat,x
  sta $08
  lda BC2Xposdat,x
  sta $09

  lda $20
  clc
  adc #$03
  clc
  sbc $08
  cmp $22
  bne BC2NA
    lda $21
    clc
    adc #$07
    clc
    sbc $09
    cmp $23
    bne BC2NA
      jmp charmove
BC2NA:
  inx
  cpx #$08
  bne BC2C1L
  jmp gameloop2

BC2Yposdat:
 .db $04
 .db $04
 .db $03
 .db $03
 .db $01
 .db $01
 .db $00
 .db $00

BC2Xposdat: 
 .db $07
 .db $05
 .db $08 
 .db $04
 .db $08
 .db $04
 .db $05
 .db $07

blackgameloop2J3:
 jmp gameloop2
blackcNOT3J:
 jmp blackcNOT3
blackcNOT2:
 cmp #$0B
 bne blackcNOT3J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  bpl BC3U
    bmi BC3D
      jmp gameloop2
  BC3U:
   lda $21 
   cmp $23
   bpl BC3UL
     bmi BC3UR
       jmp gameloop2
     BC3UL:
      dec $08
      dec $09
      lda $08
      cmp #$FF
      beq blackgameloop2J3
      lda $09
      cmp #$FF
      beq blackgameloop2J3
      jsr BC3C
      jmp BC3UL
     BC3UR:
      dec $08
      inc $09
      lda $08
      cmp #$FF
      beq blackgameloop2J3
      lda $09
      cmp #$08
      beq blackgameloop2J3
      jsr BC3C
      jmp BC3UR
  BC3D:
   lda $21 
   cmp $23
   bpl BC3DL
     bmi BC3DR
       jmp gameloop2
     BC3DL:
      inc $08
      dec $09
      lda $08
      cmp #$08
      beq blackgameloop2J3
      lda $09
      cmp #$FF
      beq blackgameloop2J4
      jsr WC3C
      jmp WC3DL
     BC3DR:
      inc $08
      inc $09
      lda $08
      cmp #$08
      beq blackgameloop2J4
      lda $09
      cmp #$08
      beq blackgameloop2J4
      jsr WC3C
      jmp WC3DR
       
  BC3C:
   lda $08
   cmp $22
   bne BC3CN
     lda $09
     cmp $23
     bne BC3CN
       jmp charmove
     BC3CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne blackgameloop2J4
      rts

blackgameloop2J4:
 jmp gameloop2
blackcNOT4J:
 jmp blackcNOT4
blackcNOT3:
 cmp #$0C
 bne blackcNOT4J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  beq BC4YEQ
  lda $21
  cmp $23
  beq BC4XEQ
    jmp gameloop2
 BC4YEQ:
  lda $21
  cmp $23
  bpl BC4YL
    bmi BC4YR
      jmp gameloop2
  BC4YL:
   dec $09
   lda $09
   cmp #$FF
   beq blackgameloop2J4
   jsr WC4C
   jmp BC4YL
  BC4YR:
   inc $09
   lda $09
   cmp #$08
   beq blackgameloop2J4
   jsr BC4C
   jmp BC4YR
 BC4XEQ:
  lda $20
  cmp $22
  bpl BC4XU
    bmi BC4XD
      jmp gameloop2
  BC4XU:
   dec $08
   lda $08
   cmp #$FF
   beq blackgameloop2J4
   jsr BC4C
   jmp BC4XU
  BC4XD:
   inc $08
   lda $08
   cmp #$08
   beq blackgameloop2J4
   jsr BC4C
   jmp BC4XD
  
  BC4C:
   lda $08
   cmp $22
   bne BC4CN
     lda $09
     cmp $23
     bne BC4CN
       jmp charmove
     BC4CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne blackgameloop2J5
      rts

blackgameloop2J5:
 jmp gameloop2
blackcNOT5J:
 jmp blackcNOT5

blackcNOT4:
 cmp #$0D
 bne blackcNOT5J

  lda $20
  sta $08
  lda $21
  sta $09

  lda $20
  cmp $22
  beq BC5YEQ
  lda $21
  cmp $23
  beq BC5XEQ
  lda $20
  cmp $22
  bpl BC5U
    bmi BC5DJ
      jmp gameloop2
BC5DJ:
 jmp BC5D

 BC5YEQ:
  lda $21
  cmp $23
  bpl BC5YL
    bmi BC5YR
      jmp gameloop2
  BC5YL:
   dec $09
   lda $09
   cmp #$FF
   beq blackgameloop2J5
   jsr BC5C
   jmp BC5YL
  BC5YR:
   inc $09
   lda $09
   cmp #$08
   beq blackgameloop2J5
   jsr BC5C
   jmp BC5YR
 BC5XEQ:
  lda $20
  cmp $22
  bpl BC5XU
    bmi BC5XD
      jmp gameloop2
  BC5XU:
   dec $08
   lda $08
   cmp #$FF
   beq blackgameloop2J5
   jsr BC5C
   jmp BC5XU
  BC5XD:
   inc $08
   lda $08
   cmp #$08
   beq blackgameloop2J6
   jsr BC5C
   jmp BC5XD
  
blackgameloop2J6:
  jmp gameloop2

 BC5U:
  lda $21 
  cmp $23
  bpl BC5UL
    bmi BC5UR
      jmp gameloop2
    BC5UL:
      dec $08
      dec $09
      lda $08
      cmp #$FF
      beq blackgameloop2J6
      lda $09
      cmp #$FF
      beq blackgameloop2J6
      jsr BC5C
      jmp BC5UL
    BC5UR:
      dec $08
      inc $09
      lda $08
      cmp #$FF
      beq blackgameloop2J6
      lda $09
      cmp #$08
      beq blackgameloop2J6
      jsr BC5C
      jmp BC5UR
 BC5D:
  lda $21 
  cmp $23
  bpl BC5DL
    bmi BC5DR
      jmp gameloop2
    BC5DL:
     inc $08
     dec $09
     lda $08
     cmp #$08
     beq blackgameloop2J6
     lda $09
     cmp #$FF
     beq blackgameloop2J6
     jsr BC5C
     jmp BC5DL
    BC5DR:
     inc $08
     inc $09
     lda $08
     cmp #$08
     beq blackgameloop2J6
     lda $09
     cmp #$08
     beq blackgameloop2J6
     jsr BC5C
     jmp BC5DR
  
  BC5C:
   lda $08
   cmp $22
   bne BC5CN
     lda $09
     cmp $23
     bne BC5CN
       jmp charmove
     BC5CN:
      jsr CPdataget
      lda $0B
      and #%00000111
      cmp #$00
      bne blackgameloop2J7
      rts
 
blackgameloop2J7:
  jmp gameloop2
blackcNOT5:  
 cmp #$0E
 bne charmove

   lda $20
   clc
   sbc $22
   cmp #$FE
   beq BC7YA
   cmp #$FF
   beq BC7YA
   cmp #$00
   beq BC7YA
     jmp gameloop2
     
   BC7YA:
     lda $21
     clc
     sbc $23
     cmp #$FE
     beq charmove
     cmp #$FF 
     beq charmove
     cmp #$00
     beq charmove
       jmp gameloop2

charmove:
 lda $01
 ldx $02
 sta $40,x
 lda #$00
 ldx $00
 sta $40,x
 lda $18
 cmp #$00
 bne pcwhite
   jmp Bgameturn
pcwhite:
   jmp Wgameturn

gamemenu:
 lda #$21
 sta $00
 lda #$02
 sta $01
 lda #$28
 sta $02
 lda #$2C
 sta $03
 jsr word

 lda #$21
 sta $00
 lda #$42
 sta $01
 lda #$1B
 sta $02
 lda #$1F
 sta $03
 jsr word

 lda #$21
 sta $00
 lda #$82
 sta $01
 lda #$2F
 sta $02
 lda #$33
 sta $03
 jsr word

 lda #$20
 sta $2006
 lda #$C0
 sta $2006
 jsr drawbox

 jsr gamemenucurset

gmloop:
 lda #$01
 sta $00
 jsr getbutton
 cmp #$01
 beq gmmodeselect
 lda #$02
 sta $00
 jsr getbutton
 cmp #$01
 beq gmreturn
 lda #$04
 sta $00
 jsr getbutton
 cmp #$01
 beq gmmodeselect
; lda #$03
; sta $00
; jsr getbutton
; cmp #$01
; beq gmreturn
 lda #$05
 sta $00
 jsr getbutton
 cmp #$01
 beq gmup
 lda #$06
 sta $00
 jsr getbutton
 cmp #$01
 beq gmdown
 jmp gmloop
gmup:
 ldx $1A
 dex
 cpx #$FF
 bne gmok
 ldx #$00
 jmp gmloop
gmdown:
 ldx $1A 
 inx
 cpx #$03
 bne gmok
 jmp gmloop
gmok:
 stx $1A
 jsr gamemenucurset
 lda #$60
 sta $00
 jsr pause
 jmp gmloop
gmmodeselect:
 jmp loop
gmreturn:
 jsr vv
 lda #$00
gmretcl:
 ldy #$21
 sty $2006
 sta $2006
 ldy #$00
 ldx #$00
gmretcll:
 sty $2007
 inx
 cpx #$08
 bne gmretcll
 clc
 adc #$20
 cmp #$00
 bne gmretcl

 jmp gameloop

gamemenucurset:    ; Code for drawing position of cursor      
 ldx #$00               ; LOW VRAM adress 
 ldy #$00               ; Compare value
gmcurset:
 jsr vv
 lda #$21               ; HIGH VRAM adress
 sta $2006              
 stx $2006
 lda $1A                ; Cursor position
 sty $00               
 cmp $00                ; Compare cursor position -2 with counter
 beq gmcurhere          ; BEQ if they are equal.
 lda #$00               ; If not, draw it blank
 sta $2007
 jmp gmcurfinishdraw
gmcurhere:
 lda #$77               ; $62 = Cursor tile
 sta $2007
gmcurfinishdraw:
 txa                    ; Make operation possible
 clc
 adc #$40               ; Add $40 to LOW VRAM adress
 tax                    ; (+2 lines)
 iny
 cpy #$04               ; Do on three lines
 bne gmcurset
 rts
 
gameilY:
 sta $20
 ldx #$00
 jmp gameilYmloop
gameilYmloop: 
 lda #$10
 jsr gameilYmloopadr
 lda #$14
 jsr gameilYmloopadr
 lda #$18
 jsr gameilYmloopadr
 lda #$1C
 jsr gameilYmloopadr
 inx
 stx $01
 lda #$08
 sta $00
 jsr pause
 ldx $01
 cpx #$10
 bne gameilYmloop
 rts

gameilYmloopadr:
 sta $2003
 ldy $2004
 sta $2003 
 lda $02
 cmp #$00
 bne gameilYmloopadrIN
 dey
 jmp gameilYmloopadrF
gameilYmloopadrIN:
 iny
gameilYmloopadrF:
 sty $2004
 rts

gameilX:
 sta $21
 ldx #$00
 jmp gameilXmloop
gameilXmloop: 
 lda #$13
 jsr gameilXmloopadr
 lda #$17
 jsr gameilXmloopadr
 lda #$1B
 jsr gameilXmloopadr
 lda #$1F
 jsr gameilXmloopadr
 inx
 stx $01
 lda #$08
 sta $00
 jsr pause
 ldx $01
 cpx #$10
 bne gameilXmloop
 rts

gameilXmloopadr:
 sta $2003
 ldy $2004
 sta $2003 
 lda $02
 cmp #$00
 bne gameilXmloopadrIN
 dey
 jmp gameilXmloopadrF
gameilXmloopadrIN:
 iny
gameilXmloopadrF:
 sty $2004
 rts

gameilY2:
 sta $22
 ldx #$00
 jmp gameilYmloop2
gameilYmloop2: 
 lda #$00
 jsr gameilYmloopadr2
 lda #$04
 jsr gameilYmloopadr2
 lda #$08
 jsr gameilYmloopadr2
 lda #$0C
 jsr gameilYmloopadr2
 inx
 stx $01
 lda #$08
 sta $00
 jsr pause
 ldx $01
 cpx #$10
 bne gameilYmloop2
 rts

gameilYmloopadr2:
 sta $2003
 ldy $2004
 sta $2003 
 lda $02
 cmp #$00
 bne gameilYmloopadrIN2
 dey
 jmp gameilYmloopadrF2
gameilYmloopadrIN2:
 iny
gameilYmloopadrF2:
 sty $2004
 rts

gameilX2:
 sta $23
 ldx #$00
 jmp gameilXmloop2
gameilXmloop2: 
 lda #$03
 jsr gameilXmloopadr2
 lda #$07
 jsr gameilXmloopadr2
 lda #$0B
 jsr gameilXmloopadr2
 lda #$0F
 jsr gameilXmloopadr2
 inx
 stx $01
 lda #$08
 sta $00
 jsr pause
 ldx $01
 cpx #$10
 bne gameilXmloop2
 rts

gameilXmloopadr2:
 sta $2003
 ldy $2004
 sta $2003 
 lda $02
 cmp #$00
 bne gameilXmloopadrIN2
 dey
 jmp gameilXmloopadrF2
gameilXmloopadrIN2:
 iny
gameilXmloopadrF2:
 sty $2004
 rts

CPU:
 jmp loop

mcursprallset:
 lda #$00
 clc 
 adc #$40
 ldx #$00
mcsasYcheck:
 clc
 adc #$08
 inx
 cpx $22
 bne mcsasYcheck
 ldx #$00
mcsasXcheck:
 clc
 adc #$01
 inx
 cpx $23
 bne mcsasXcheck
 tax
 lda $00,x
 and #%00000111
 cmp #$00
 bne mcsprpalchar
 lda #$6A
 sta $01
 lda #$2F
 sta $02
 lda #$47
 sta $03
 lda #$02
 sta $06
 jmp mcsprpalf
mcsprpalchar:
 lda $00,x
 and #%00001000
 lsr A
 lsr A
 lsr A
 cmp $18
 bne mcsprpalnot0
 lda #$6C
 sta $01
 lda #$37
 sta $02
 lda #$40
 sta $03
 lda #$01
 sta $06
 jmp mcsprpalf
mcsprpalnot0:
 lda #$6E
 sta $01
 lda #$2F
 sta $02
 lda #$47
 sta $03
 lda #$03
 sta $06
mcsprpalf:
 ldx #$FF
 ldy #$02
 
 lda $02
 sta $04
 lda $01
 sta $05
 lda $03
 sta $07
 jsr cursprset

 lda $02
 sta $04
 lda $01
 sta $05
 inc $05
 lda $03
 clc
 adc #$08
 sta $03
 sta $07
 jsr cursprset

 lda $02
 clc
 adc #$08
 sta $02
 sta $04
 lda $01
 clc
 adc #$10
 sta $05
 lda $03
 clc
 sbc #$07
 sta $03
 sta $07
 jsr cursprset

 lda $02
 sta $04
 lda $01
 clc
 adc #$11
 sta $05
 lda $03
 clc
 adc #$08
 sta $03
 sta $07
 jsr cursprset
 rts

cursprallset:
 ldx #$0F
 ldy #$00
 sty $06

 lda #$36
 sta $04
 lda #$68
 sta $05
 lda #$3F
 sta $07
 jsr cursprset

 lda #$36
 sta $04
 lda #$69
 sta $05
 lda #$49
 sta $07
 jsr cursprset

 lda #$40
 sta $04
 lda #$78
 sta $05
 lda #$3F
 sta $07
 jsr cursprset

 lda #$40
 sta $04
 lda #$79
 sta $05
 lda #$49
 sta $07
 jsr cursprset
 rts

cursprset:           
 inx
 stx $2003
 lda #$00
 clc
 adc $04
 sta $00
 lda $20,y
 asl A
 asl A
 asl A
 asl A
 clc
 adc $00
 sta $2004
 inx
 stx $2003
 lda $05
 sta $2004
 inx
 stx $2003
 lda $06
 sta $2004
 inx
 stx $2003
 lda #$00
 clc
 adc $07
 sta $00
 lda $21,y
 asl A
 asl A
 asl A
 asl A
 clc
 adc $00
 sta $2004
 rts

chessboardreset:
 lda #$05
 sta $8001
 lda #$02
 sta $8002
 lda #$01
 sta $8005

 jsr clearsprite

 jsr vv
 jsr nametableadressing
 lda #$00
 ldx #$00
cbntres:
 sta $2007
 inx 
 cpx #$00
 bne cbntres
 
 jsr vv
 lda #$22
 sta $2006
 lda #$E0
 sta $2006

cbntres2: 
 ldy #$39  
 sty $2007
 sty $2007
 dey
 sty $2007
 sty $2007
 inx
 cpx #$06
 bne cbntres2
 
 lda #$00
 ldx #$00
cbntres3:
 sta $2007
 inx     
 cpx #$A8
 bne cbntres3

 lda #$3F
 sta $2006
 lda #$14
 sta $2006
 ldx #$00
gamesprpalset:
 lda gamesprpal,x
 sta $2007
 inx
 cpx #$0C
 bne gamesprpalset

 jsr vv
 jsr attableadressing
 lda $00
 ldx #$00
cbasloop:
 sta $2007
 inx
 cpx #$40
 bne cbasloop
 rts

chessboardset:
 ldx #$00
chessboardsetloop:
 txa
 and #%00000001
 sta $00
 txa
 and #%00001000
 cmp #$00
 bne cbdlinenot0 
   lda $00
   cmp #$00
   bne cbdline0dotnot0
     lda #$80
     sta $01
     jmp cbdfdotset
   cbdline0dotnot0:
     lda #$C0
     sta $01
     jmp cbdfdotset
 cbdlinenot0:
   lda $00
   cmp #$00
   bne cbdline1dotnot0
     lda #$C0
     sta $01
     jmp cbdfdotset
    cbdline1dotnot0:
     lda #$80
     sta $01
     jmp cbdfdotset
cbdfdotset:
 lda $40,x
 and #%00001000
 asl A
 asl A
 clc
 adc $01
 sta $01
 lda $40,x
 and #%00000111
 asl A
 asl A
 clc
 adc $01
 sta $01
 lda $01
 sta $80,x
 inx   
 cpx #$40
 bne chessboardsetloop
 rts

chessboarddraw:
 jsr vv
 lda #$20
 sta $2006
 lda #$E8
 sta $2006
 ldy #$00
cbdnextstream:
 ldx #$00 
cbdnextpoint:
 lda $80,y
 sta $2007
 sta $00
 inc $00
 lda $00
 sta $2007
 iny
 inx
 cpx #$08
 bne cbdnextpoint
 
 lda #$00
 ldx #$00
cbdnextspacetile: 
 sta $2007
 inx
 cpx #$10
 bne cbdnextspacetile
 
 ldx #$00
cbdnextDEY: 
 dey
 inx
 cpx #$08
 bne cbdnextDEY
 
 ldx #$00
cbdnextpointl:
 lda $80,y
 sta $00
 inc $00
 inc $00
 lda $00
 sta $2007
 sta $00
 inc $00
 lda $00
 sta $2007
 iny
 inx
 cpx #$08
 bne cbdnextpointl

 lda #$00
 ldx #$00
cbdnextspacetilel: 
 sta $2007
 inx
 cpx #$10
 bne cbdnextspacetilel

 cpy #$40
 bne cbdnextstream

 jsr vv
 lda #$1A
 sta $00
 lda #$04
 sta $01
 lda $18
 asl A
 asl A
 asl A
 asl A
 sta $03
 lda #$40
 clc
 adc $03
 sta $02

 ldx #$00
 ldy #$00
cbtclinedrw: 
 lda #$21
 sta $2006
 lda $00
 sta $2006
cbtctiledrw:
 lda $02
 sta $2007
 inc $02  
 inx
 cpx $01
 bne cbtctiledrw
 iny
 lda $01
 clc
 adc #$04
 sta $01
 lda $00
 clc
 adc #$20
 sta $00
 cpy #$04
 bne cbtclinedrw
 
 rts

 .bank 3
 .org $F800
gamesprpal: .db $0F,$0F,$16,$06,$0F,$0F,$2A,$1A,$0F,$0F,$28,$18




 .bank 3
 .org $FC00
word:
 jsr vv
 lda $00
 sta $2006
 lda $01
 sta $2006
 ldx $02
wordloop:
 lda text0,x
 sta $2007
 inx
 cpx $03
 bne wordloop
 rts

text:  
introtxtnextchar:
; lda introtxt0,x
 cmp #$F1
 beq introtxtstep
 cmp #$F2
 beq introtxtend
 cmp #$F3
 beq introtxtpreset
 jsr vv
 sta $2007
 inx
 jmp introtxtnextchar
introtxtstep:
 inx
; lda introtxt0,x
 inx
 sta $00
 lda #$00
 ldy #$00
introtxtstepnext:     
 jsr vv
 sta $2007
 iny
 cpy $00
 bne introtxtstepnext
 jmp introtxtnextchar
introtxtpreset:
 inx
; lda introtxt0,x
 sta $00
 inx 
; lda introtxt0,x
 sta $01
 inx  
 ldy #$00
 lda $00
introtxtpresetnext:
 jsr vv
 sta $2007
 sta $02
 inc $02
 lda $02
 iny
 cpy $01
 bne introtxtpresetnext
 jmp introtxtnextchar
introtxtend:
 rts

clearsprite:
 lda #$00
 ldx #$00
resetspriteloop:  
 stx $2003
 sta $2004
 inx
 cpx #$00
 bne resetspriteloop
 rts

drawbox:
 ldx #$00
drawboxl: 
 lda box,x
 sta $00
 inx
 lda box,x
 inx
 ldy #$00
drawboxstr:
 sta $2007
 iny
 cpy $00
 bne drawboxstr
 cpx #$1E
 bne drawboxl
 rts

clearnttable:
 jsr vv
 jsr nametableadressing
 lda #$00
 ldy #$00
clearntchunk:
 ldx #$00
clearntbyte:
 sta $2007
 inx
 cpx #$F0
 bne clearntbyte
 iny
 cpy #$04
 bne clearntchunk
 rts

loop:
 jmp loop

pause:
 ldy #$00
pauseloopY:
 ldx #$00
pauseloopX:
 nop
 nop
 inx
 cpx #$FF
 bne pauseloopX
 iny
 cpy $00
 bne pauseloopY
 rts

paladressing:
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 rts

nametableadressing:
 lda #$20
 sta $2006
 lda #$00
 sta $2006
 rts

attableadressing:
 lda #$23
 sta $2006
 lda #$C0
 sta $2006
 rts

vv:
 lda $2002
 bpl vv
 lda #$00
 sta $2005
 sta $2005
 sta $2006
 sta $2006
 rts

drawon:
 lda #%00001000
 sta $2000
 lda #%00011110
 sta $2001      
 rts

drawoff:
 lda #%00001000
 sta $2000
 lda #%00000110
 sta $2001      
 rts

center:
 sta $C0
 lda #$00
 sta $2005
 sta $2005
 lda $C0
 rts

getbutton:
 ldy $16
 lda #$01
 sta $4016,y
 tax
 dex
 txa
 sta $4016,y
 ldx #$00
getbuttonloop:
 lda $4016,y
 inx
 cpx $00
 bne getbuttonloop
 and #$01
 rts

CPdataget:
 lda #$00
 ldx #$00
cpdYcount:
 clc
 adc #$08
 inx
 cpx $08
 bne cpdYcount
 ldx #$00
cpdXcount:
 clc
 adc #$01
 inx
 cpx $09
 bne cpdXcount
 sta $0A
 tax
 lda $40,x
 sta $0B
 rts

text0: 
  .db B,xY,0,$25,$26,$27,$28,$29,$2A,$2B
  .db W,H,I,T,E
  .db $30,$31,$32,$33,$34
  .db T,E,xX,T,U,R,E
  .db L,O,xA,D,0,G,xA,M,E
  .db T,U,R,N
  .db S,xA,V,E
  .db N,E,W
  .db E,xX,I,T
box: 
  .db $01,$60,$02,$61,$01,$68,$0C,$61,$01,$62,$0F,$00
  .db $01,$63,$02,$6A,$01,$78,$0C,$00,$01,$64,$0F,$00
  .db $01,$65,$0F,$66,$01,$67

nmi:
 rti




 .bank 3
 .org $FFFA
 .dw nmi
 .dw program
 .bank 4
 .incbin "CHESS.CHR"
